@@ -14,6 +14,8 @@
 #include <QDesktopServices>
 #include <QDebug>
 #include <openssl/evp.h>
+#include <openssl/sha.h>
+
 
 #define nBASE64
 
@@ -132,14 +134,12 @@ DatabaseManager::DatabaseManager(QObject *parent) :
 
 DatabaseManager::~DatabaseManager()
 {
-//	QString path(QDir::home().path());
-//	path.append(QDir::separator()).append("BACKqtwallet.db.sqlite");
-//	path = QDir::toNativeSeparators(path);
-//	//db.setDatabaseName(path);
-//
-//	QSqlDatabase::cloneDatabase(db, path);
+	if (connectionName.length() > 0)
+		db.removeDatabase(connectionName);
+//	else
+//		db.removeDatabase();
 
-	qDebug () << "CLOSE DB";
+	qDebug () << "CLOSE DB " << connectionName;
 	if (db.isOpen()) {
 		qDebug () << "CLOSing DB";
 		db.close();
@@ -156,7 +156,7 @@ bool DatabaseManager::isInit()
 
 bool DatabaseManager::initDB()
 {
-	qDebug () << "INIT DB";
+	qDebug () << "INIT DB " << connectionName;
 	bool ret = true;
 
 	// Create tables
@@ -164,55 +164,43 @@ bool DatabaseManager::initDB()
 		createTargetTable();
 		createTargetFieldTable();
 		createTableVersion ();
+		ret = addVersion (SOFT_VERSION);
 	}
+	currentVersion = getVersion ();
 
 	// Check that tables exists
-			if (db.tables().count() != 4)
-				ret = false;
+	if (db.tables().count() != 4)
+		ret = false;
 
 	return ret;
 }
 
-//bool copyFile(QString oldFilePath, QString newFilePath)
-//{
-//  //same file, no need to copy
-//  if(oldFilePath.compare(newFilePath) == 0)
-//    return true;
-//
-//  //load both files
-//  QFile oldFile(oldFilePath);
-//  QFile newFile(newFilePath);
-//  bool openOld = oldFile.open( IO_ReadOnly );
-//  bool openNew = newFile.open( IO_WriteOnly );
-//
-//  //if either file fails to open bail
-//  if(!openOld || !openNew) { return false; }
-//
-//  //copy contents
-//  uint BUFFER_SIZE = 16000;
-//  char* buffer = new char[BUFFER_SIZE];
-//  while(!oldFile.atEnd())
-//  {
-//    Q_ULONG len = oldFile.readBlock( buffer, BUFFER_SIZE );
-//    newFile.writeBlock( buffer, len );
-//  }
-//
-//  //deallocate buffer
-//  delete[] buffer;
-//  buffer = NULL;
-//  return true;
-//}
-
-bool DatabaseManager::migrateVersion (int fromV, int toV) {
-	bool ret = false;
+bool DatabaseManager::migrateVersion (int fromV, int toV, const QString & pas) {
+	bool ret = true;
 
 	qDebug () << "MIGRATING DB VERSION FROM " << fromV << " TO " << toV;
 
 	if (fromV == 0 && toV == 1) {
-		backupDB ();
+		if (backupDB ("BACK")) {
+			QSqlQuery query(db);
+			ret = query.exec("alter table targets ADD COLUMN groupName varchar(100)");
+		}
+		else
+			ret = false;
+	}
 
-		QSqlQuery query;
-		ret = query.exec("alter table targets ADD COLUMN groupName varchar(100)");
+	if (fromV == 1 && toV == 2) {
+		if (backupDB ("BACK")) {
+			if (createPassword(pas, 2)) {
+				if (!addVersion(2)) {
+					ret = false;
+				}
+			}
+			else
+				ret = false;
+		}
+		else
+			ret = false;
 	}
 
 	qDebug () << "MIGRATED DB VERSION FROM " << fromV << " TO " << toV << " WITH RESULT " << ret;
@@ -220,54 +208,131 @@ bool DatabaseManager::migrateVersion (int fromV, int toV) {
 	return ret;
 }
 
-bool DatabaseManager::deleteDB()
+bool DatabaseManager::deleteDB(const QString & prefix)
 {
-	qDebug () << "REMOVE DB";
+	qDebug () << "REMOVE DB " << connectionName;
 	db.close();
 
 	QString path(QDir::home().path());
-	path.append(QDir::separator()).append("qtwallet.db.sqlite");
+	path.append(QDir::separator()).append(prefix).append("qtwallet.db.sqlite");
 	path = QDir::toNativeSeparators(path);
 
 	return QFile::remove(path);
 }
-bool DatabaseManager::backupDB() {
+bool DatabaseManager::backupDB(const QString & prefix) {
 	QString path(QDir::home().path());
 	path.append(QDir::separator()).append("qtwallet.db.sqlite");
 	path = QDir::toNativeSeparators(path);
 
 	QString pathBack(QDir::home().path());
-	pathBack.append(QDir::separator()).append("BACKqtwallet.db.sqlite");
+	pathBack.append(QDir::separator()).append(prefix).append("qtwallet.db.sqlite");
 	pathBack = QDir::toNativeSeparators(pathBack);
 
-	QFile file (path);
-	bool dup = file.copy(pathBack);
+	bool dup = true;
 
-	qDebug () << "duplicate DB " << dup;
+	QFile fileBack (pathBack);
+	if (fileBack.exists())
+		dup = fileBack.remove();
+
+	if (dup) {
+		QFile file (path);
+		dup = file.copy(pathBack);
+	}
+
+	qDebug () << "duplicate " << prefix << " DB " << dup;
 
 	return dup;
 }
-bool DatabaseManager::openDB()
-{
-	qDebug () << "OPEN DB";
-	// Find QSLite driver
-	db = QSqlDatabase::addDatabase("QSQLITE");
-	// http://doc.trolltech.com/4.5/sql-driver.html#qsqlite
-
+bool DatabaseManager::restoreDB(const QString & prefix) {
 	QString path(QDir::home().path());
 	path.append(QDir::separator()).append("qtwallet.db.sqlite");
 	path = QDir::toNativeSeparators(path);
 
+	QString pathBack(QDir::home().path());
+	pathBack.append(QDir::separator()).append(prefix).append("qtwallet.db.sqlite");
+	pathBack = QDir::toNativeSeparators(pathBack);
+
+	bool dup = true;
+
+	QFile file (path);
+	if (file.exists())
+		dup = file.remove();
+
+	if (dup) {
+		QFile fileBack (pathBack);
+		dup = fileBack.copy(path);
+	}
+
+	qDebug () << "restore " << prefix << " DB " << dup;
+
+	return dup;
+}
+//bool DatabaseManager::openDB()
+//{
+//	return _openDB ("qtwallet.db.sqlite");
+//}
+
+bool DatabaseManager::migrationFrom (DatabaseManager * mgr) {
+	bool res;
+
+    QList<Target *> list = mgr->getTargets(NULL);
+
+	for (int i = 0; i < list.size (); i++) {
+		Target * target = list[i];
+
+		qDebug () << "MIGRATING " << target->name;
+
+		mgr->getFields(target);
+
+		res = insertTarget(target);
+
+		qDebug () << "MIGRATING 1 " << target->name << res;
+		if (!res) return false;
+
+		if (target->getPicture() != NULL) {
+			res = updateTargetPicture (target);
+			qDebug () << "MIGRATING 2 " << target->name << res;
+			if (!res) return false;
+		}
+
+		if (target->getPictureBack() != NULL) {
+			res = updateTargetPictureBack (target);
+			qDebug () << "MIGRATING 3 " << target->name << res;
+			if (!res) return false;
+		}
+
+		res = updateFields (target);
+		qDebug () << "MIGRATING 4 " << target->name << res;
+		if (!res) return false;
+
+		target->freeFieldList();
+		target->freeResources();
+	}
+
+	return res;
+}
+
+bool DatabaseManager::_openDB(const QString & name, const QString & cName) {
+	qDebug () << "OPEN DB " << name << "(" << cName << ")";
+
+	connectionName = cName;
+
+	// Find QSLite driver
+	if (connectionName.length() > 0)
+		db = QSqlDatabase::addDatabase("QSQLITE", connectionName);
+	else
+		db = QSqlDatabase::addDatabase("QSQLITE");
+	// http://doc.trolltech.com/4.5/sql-driver.html#qsqlite
+	QString path(QDir::home().path());
+	path.append(QDir::separator()).append(name);
+	path = QDir::toNativeSeparators(path);
+
 	db.setDatabaseName(path);
 
 	// Open databasee
-	if (db.open()) {
-		// Check that tables exists
-		if (db.tables().count() == 3) {
-			migrateVersion (0, 1);
-			return createTableVersion ();
-		}
+	bool res = false;
 
+	if (db.open()) {
 		return true;
 	}
 	return false;
@@ -283,7 +348,7 @@ bool DatabaseManager::createIdTable()
 	// Create table
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.exec("create table id (id integer primary key, phrase varchar(20))");
 	}
 	return ret;
@@ -294,16 +359,30 @@ bool DatabaseManager::createTableVersion()
 	// Create table
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.exec("create table softwareVersion (versionNumber integer)");
+	}
+	return ret;
+}
+int DatabaseManager::getVersion() {
+	if (db.isOpen()) {
+		QSqlQuery query (db);
+		bool ret = query.exec("select max(versionNumber) from softwareVersion");
 		if (ret) {
-			ret = query.prepare("insert into softwareVersion values (:ver)");
-			if (ret) {
-				query.bindValue(":ver", SOFT_VERSION);
-				ret = query.exec();
+			if (query.next()) {
+				return query.value(0).toInt();
 			}
 		}
 	}
+	return -1;
+}
+bool DatabaseManager::addVersion(int v) {
+	QSqlQuery query (db);
+	bool ret = query.prepare("insert into softwareVersion values (:ver)");
+	if (ret) {
+		query.bindValue(":ver", v);
+		ret = query.exec();
+	}
 	return ret;
 }
 
@@ -311,7 +390,7 @@ int DatabaseManager::nextId()
 {
 	int ret = -1;
 	if (db.isOpen()) {
-		QSqlQuery query("select id from id");
+		QSqlQuery query("select id from id", db);
 		if (query.next()) {
 			// Get last used id
 			ret = query.value(0).toInt();
@@ -334,7 +413,7 @@ int DatabaseManager::nextId()
 void DatabaseManager::reset () {
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret |= query.exec("drop table id");
 		ret |= query.exec("drop table targets");
 		ret |= query.exec("drop table fieds");
@@ -343,43 +422,110 @@ void DatabaseManager::reset () {
 
 }
 
+QString DatabaseManager::changePassword (const QString & newPassword) {
+	bool ret = false;
+
+	QString result = "";
+
+	ret = backupDB("BACK");
+
+	if (ret) {
+		DatabaseManager * oldDB = new DatabaseManager();
+		ret = oldDB->_openDB("NEWqtwallet.db.sqlite", "NEW");
+		if (ret) {
+			oldDB->reset ();
+			ret = oldDB->initDB();
+			if (ret){
+				ret = oldDB->createPassword(newPassword, oldDB->currentVersion);
+				if (ret) {
+					if (oldDB->migrationFrom(this)) {
+						qDebug () << "succesfull migrated ";
+						db.close ();
+
+						ret = restoreDB ("NEW");
+						if (ret) {
+							qDebug () << "succesfull changed ";
+
+						}
+						else {
+							result = "DB cannot be restored";
+						}
+
+					}
+					else
+						result = "NEWqtwallet.db.sqlite cannot be migrated";
+				}
+				else
+					result = "NEWqtwallet.db.sqlite cannot be init";
+			}
+			else
+				result = "NEWqtwallet.db.sqlite cannot be reset";
+		}
+		else
+			result = "NEWqtwallet.db.sqlite cannot be opened";
+		delete oldDB;
+	}
+
+	return result;
+}
+
+
 void DatabaseManager::lock () {
 	initdb = false;
 }
 
 bool DatabaseManager::hasPassword() {
 	if (!opendb)
-		opendb = openDB();
+		opendb = _openDB("qtwallet.db.sqlite");
 
 	return opendb && isInit () && hasSecretPhrase();
 }
 
-bool DatabaseManager::openDB(QString pas) {
+bool DatabaseManager::openDB(const QString & pas) {
 	initdb = false;
 
-	qDebug() << "open db with pas";
+	qDebug() << "open db with pas " << SOFT_VERSION;
 
 	if (!opendb)
-		opendb = openDB();
+		opendb = _openDB("qtwallet.db.sqlite");
 
 	if (!opendb) return false;
 
 	qDebug() << "init db with pas";
 
-	if (isInit ()) {
-		if (hasSecretPhrase()) {
-			initdb = checkPassword(pas);
-		}
-		else {
-			initdb = createPassword(pas);
-		}
+	if (db.tables().count() == 0) {
+		if (initDB())
+			initdb = createPassword(pas, SOFT_VERSION);
+		currentVersion = SOFT_VERSION;
 	}
 	else {
-		if (initDB())
-			initdb = createPassword(pas);
+		bool res = false;
+		// Check that tables exists
+		if (db.tables().count() == 3) {
+			res = migrateVersion (0, 1, pas);
+			res = createTableVersion ();
+			addVersion (1);
+			currentVersion = 1;
+		}
+
+		if (db.tables().count() != 4) {
+			initdb = false;
+			currentVersion = SOFT_VERSION;
+		}
+		else {
+			res = migrateVersion (getVersion (), SOFT_VERSION, pas);
+			currentVersion = getVersion ();
+
+			if (hasSecretPhrase()) {
+				initdb = checkPassword(pas, currentVersion);
+			}
+			else {
+				initdb = createPassword(pas, currentVersion);
+			}
+		}
 	}
 
-	qDebug() << "init db with pas " << initdb;
+	qDebug() << "init db with pas " << initdb << " " << currentVersion;
 
 	return initdb;
 }
@@ -388,9 +534,9 @@ bool DatabaseManager::isReady () {
 	return opendb && initdb;
 }
 
-bool DatabaseManager::checkPassword (QString _a)
+bool DatabaseManager::checkPassword (QString _a, int version)
 {
-	qDebug () << "checkPassword "; // << _a;
+	qDebug () << "checkPassword " << version << "(" << connectionName << ")"; // << _a;
 
 	QByteArray t = _a.toLocal8Bit();
 	
@@ -409,17 +555,24 @@ bool DatabaseManager::checkPassword (QString _a)
 	AES_set_decrypt_key(key, keyLen, &aeskeyD);
 	
 	QByteArray a;
-	encrypt(QString("developped by nowhereman"), a);
+	if (version < 2)
+		encrypt(QString(PASS1), a);
+	else {
+		QString pp(_a);
+		pp.append (PASS2);
+		encryptSHA1(pp.mid(0, SHA1_LEN), a);
+	}
+
 	QByteArray b = selectSecretPhrase ();
 	
-	//qDebug()<< "checkPassword " << (a == b);
+	//qDebug()<< "checkPassword [[[[[" << b << "]]]]]";
 	
 	return a == b;
 }
 
-bool DatabaseManager::createPassword (QString _a)
+bool DatabaseManager::createPassword (QString _a, int version)
 {
-	qDebug () << "createPassword";
+	qDebug () << "createPassword " << version << "(" << connectionName << ")";
 
 	QByteArray t = _a.toLocal8Bit();
 	
@@ -438,7 +591,13 @@ bool DatabaseManager::createPassword (QString _a)
 	AES_set_decrypt_key(key, keyLen, &aeskeyD);
 	
 	QByteArray a;
-	encrypt(QString("developped by nowhereman"), a);
+	if (version < 2)
+		encrypt(QString(PASS1), a);
+	else {
+		QString pp(_a);
+		pp.append (PASS2);
+		encryptSHA1(pp.mid(0, SHA1_LEN), a); // encrypt
+	}
 	
 	//qDebug () << "create secret " << a;
 	
@@ -450,7 +609,7 @@ bool DatabaseManager::updateSecretPhrase(QByteArray a) {
 
 	bool ret = false;
 	if (nextId () > 0) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("update id set phrase = :phrase");
 		if (ret) {
 			query.bindValue(":phrase", a);
@@ -466,11 +625,11 @@ bool DatabaseManager::hasSecretPhrase()
 {
 	qDebug () << "has secret?";
 	
-	QSqlQuery query("select phrase from id");
+	QSqlQuery query("select phrase from id", db);
 	if (query.next()) {
 		return !query.value(0).isNull();
 	}
-
+	//query.clear();
 	return false;
 }
 
@@ -479,7 +638,7 @@ QByteArray DatabaseManager::selectSecretPhrase()
 	qDebug () << "get secret?";
 
 	QByteArray ret;
-	QSqlQuery query("select phrase from id");
+	QSqlQuery query("select phrase from id", db);
 	if (query.next()) {
 		return query.value(0).toByteArray();
 	}
@@ -492,7 +651,7 @@ bool DatabaseManager::createTargetTable()
 	// Create table
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.exec("create table targets "
 		                 "(id integer primary key, " // this is autoincrement field http://www.sqlite.org/autoinc.html
 		                 "name varchar(30), "
@@ -509,7 +668,7 @@ bool DatabaseManager::createTargetFieldTable()
 	// Create table
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.exec("create table fields "
 		                 "(id integer primary key, " // this is autoincrement field http://www.sqlite.org/autoinc.html
 		                 "targetId integer, "
@@ -523,6 +682,25 @@ bool DatabaseManager::createTargetFieldTable()
 	return ret;
 }
 
+void DatabaseManager::encryptSHA1 (const QString & a, QByteArray& b) {
+	QByteArray t = a.toLocal8Bit();
+	int length = t.size();
+
+	SHA512_CTX context;
+	unsigned char md[SHA512_DIGEST_LENGTH];
+
+	SHA512_Init(&context);
+	SHA512_Update(&context, (unsigned char*)t.constData(), length);
+	SHA512_Final(md, &context);
+
+#ifdef BASE64
+	b = QByteArray((char*)md, SHA512_DIGEST_LENGTH).toBase64();
+#else
+	b = QByteArray((char*)md, SHA512_DIGEST_LENGTH);
+#endif
+}
+
+
 void DatabaseManager::encrypt (const QString a, QByteArray& b)
 {
 	unsigned char ivE[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
@@ -625,7 +803,7 @@ bool DatabaseManager::insertTarget(Target* target)
 
 		// http://www.sqlite.org/autoinc.html
 		// NULL = is the keyword for the autoincrement to generate next value
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("INSERT INTO targets (name, type, picture, pictureBack, groupName) VALUES (:name, :type, :picture, :pictureBack, :groupName)");
 		QByteArray t1;
 
@@ -685,7 +863,7 @@ QList<Target*> DatabaseManager::getTargets(const QString & filter)
 {
 	QList<Target*> result;
 
-	QSqlQuery query;
+	QSqlQuery query (db);
 	bool ret;
 	if (filter != NULL && filter.length() > 0) {
 		ret = query.prepare("select id, name, type, groupName from targets where groupName = :groupName");
@@ -716,7 +894,7 @@ QList<QString> DatabaseManager::getGroups()
 {
 	QList<QString> result;
 
-	QSqlQuery query("select distinct coalesce(groupName,'') from targets where groupName != '' and groupName is Not Null order by groupName");
+	QSqlQuery query("select distinct coalesce(groupName,'') from targets where groupName != '' and groupName is Not Null order by groupName", db);
 	while (query.next()) {
 		result.append(query.value(0).toString());
 	}
@@ -728,7 +906,7 @@ bool DatabaseManager::deleteTarget(Target * target)
 {
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 
 		ret = query.prepare("delete from fields where targetId=:code");
 		query.bindValue(":code", target->id);
@@ -753,7 +931,7 @@ bool DatabaseManager::updateFields(Target* target)
 	bool ret = false;
 	if (db.isOpen()) {
 
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("delete from fields where targetId=:code");
 		query.bindValue(":code", target->id);
 		ret = query.exec();
@@ -779,7 +957,7 @@ bool DatabaseManager::updateTarget(Target* target)
 {
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("UPDATE targets set name=:name, type=:type, groupName=:groupName where id=:code");
 		QByteArray t1;
 		if (ret) {
@@ -801,7 +979,7 @@ bool DatabaseManager::updateTargetPicture(Target* target)
 {
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("UPDATE targets set picture=:picture where id=:code");
 		if (ret) {
 			qDebug () << "updating image " << target->id << " " << target->picture;
@@ -837,7 +1015,7 @@ bool DatabaseManager::updateTargetPictureBack(Target* target)
 {
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("UPDATE targets set pictureBack=:pictureBack where id=:code");
 		if (ret) {
 			qDebug () << "UPDATE " << target->id << " " << target->name << " " << target->type;
@@ -871,7 +1049,7 @@ bool DatabaseManager::updateTargetField(TargetField* field)
 {
 	bool ret = false;
 	if (db.isOpen()) {
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("UPDATE fields set name=:name, value=:value, type=:type, orderN=:order, "
 		                     "encrypted=:encrypted where targetId=:targetId and id=:id");
 		if (ret) {
@@ -907,7 +1085,7 @@ bool DatabaseManager::insertTargetField(TargetField* field)
 		// http://www.sqlite.org/autoinc.html
 		// NULL = is the keyword for the autoincrement to generate next value
 
-		QSqlQuery query;
+		QSqlQuery query (db);
 		ret = query.prepare("INSERT INTO fields (targetId, name, value, type, orderN, encrypted) "
 		                    "VALUES (:targetId, :name, :value, :type, :order, :encrypted)");
 		if (ret) {
@@ -948,7 +1126,7 @@ void DatabaseManager::getFields(Target * target)
 	//target->freeFieldList ();
 	
 	bool ret = false;
-	QSqlQuery query;
+	QSqlQuery query (db);
 	ret = query.prepare("select id, name, value, type, targetId, orderN, encrypted from fields where targetId=:targetId order by orderN");
 	query.bindValue(":targetId", target->id);
 	ret = query.exec ();
@@ -975,7 +1153,7 @@ void DatabaseManager::getFields(Target * target)
 
 QImage * DatabaseManager::getPicture (Target * target) {
 	QImage * result = NULL;
-	QSqlQuery query;
+	QSqlQuery query (db);
 	bool ret = query.prepare("select picture from targets where id=:targetId");
 	query.bindValue(":targetId", target->id);
 	ret = query.exec ();
@@ -1000,7 +1178,7 @@ QImage * DatabaseManager::getPicture (Target * target) {
 }
 QImage * DatabaseManager::getPictureBack (Target * target) {
 	QImage * result = NULL;
-	QSqlQuery query;
+	QSqlQuery query (db);
 	bool ret = query.prepare("select pictureBack from targets where id=:targetId");
 	query.bindValue(":targetId", target->id);
 	ret = query.exec ();
@@ -1025,3 +1203,29 @@ QImage * DatabaseManager::getPictureBack (Target * target) {
 	return result;
 }
 
+bool DatabaseManager::renameGroup (const QString & from, const QString & to) {
+	bool ret = false;
+	if (db.isOpen()) {
+		QSqlQuery query (db);
+		ret = query.prepare("UPDATE targets set groupName = :newGroupName WHERE groupName = :oldGroupName");
+		if (ret) {
+			query.bindValue(":newGroupName", to);
+			query.bindValue(":oldGroupName", from);
+			ret = query.exec ();
+		};
+	}
+	return ret;
+}
+
+bool DatabaseManager::deleteGroup (const QString & from) {
+	bool ret = false;
+	if (db.isOpen()) {
+		QSqlQuery query (db);
+		ret = query.prepare("UPDATE targets set groupName = '' WHERE groupName = :oldGroupName");
+		if (ret) {
+			query.bindValue(":oldGroupName", from);
+			ret = query.exec ();
+		};
+	}
+	return ret;
+}
