@@ -328,6 +328,7 @@ void DetailWindow::_addField (TargetField * field) {
 		QTextEdit * nameLineEdit = new QTextEdit(this);
 		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
 		nameLineEdit->setText (field->value);
+		//nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
 		this->values.append (nameLineEdit);
 		//nameLineEdit->setUserData(0,info);
 	}
@@ -336,7 +337,16 @@ void DetailWindow::_addField (TargetField * field) {
 		QLineEdit * nameLineEdit = new QLineEdit(this);
 		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
 		nameLineEdit->setText (field->value);
-		//nameLineEdit->setInputMask (">")
+		nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
+		this->values.append (nameLineEdit);
+		//nameLineEdit->setUserData(0,info);
+	}
+	else
+	if (field->type == TYPE_INTEGER) {
+		QLineEdit * nameLineEdit = new QLineEdit(this);
+		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
+		nameLineEdit->setText (field->value);
+		nameLineEdit->setInputMethodHints(Qt::ImhDigitsOnly);
 		this->values.append (nameLineEdit);
 		//nameLineEdit->setUserData(0,info);
 	}
@@ -345,8 +355,7 @@ void DetailWindow::_addField (TargetField * field) {
 		QLineEdit * nameLineEdit = new QLineEdit(this);
 		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
 		nameLineEdit->setText (field->value);
-		nameLineEdit->setInputMask ("00-00-0000");
-		nameLineEdit->setToolTip("format dd-mm-yyyy");
+		nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
 		this->values.append (nameLineEdit);
 		//nameLineEdit->setUserData(0,info);
 	}
@@ -355,16 +364,25 @@ void DetailWindow::_addField (TargetField * field) {
 		QLineEdit * nameLineEdit = new QLineEdit(this);
 		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
 		nameLineEdit->setText (field->value);
-		nameLineEdit->setInputMask ("00000000000");
+		nameLineEdit->setInputMethodHints(Qt::ImhDigitsOnly);
+		//nameLineEdit->setInputMethodHints(Qt::ImhDialableCharactersOnly);
 		this->values.append (nameLineEdit);
 
 	}
+	else
+	if (field->type == TYPE_URL) {
+		QLineEdit * nameLineEdit = new QLineEdit(this);
+		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
+		nameLineEdit->setText (field->value);
+		//nameLineEdit->setInputMethodHints(Qt::ImhUrlCharactersOnly);
+		nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
+		this->values.append (nameLineEdit);
+	}
 	else {
 		QLineEdit * nameLineEdit = new QLineEdit(this);
-		nameLineEdit->setCompleter(NULL);
-		//nameLineEdit->inputContext()->
-		//nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
 		componentLayout->addWidget(nameLineEdit, pos + 1, 3, s, 4);
+		//nameLineEdit->setCompleter(NULL);
+		nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
 		nameLineEdit->setText (field->value);
 		this->values.append (nameLineEdit);
 
