@@ -2,22 +2,16 @@
  * Copyright (c) 2010 David Galindo <nowheremanmail@gmail.com>
  *
 // TODO several files? or several wallets in database?
-// TODO export to a file with pictures!!!??
 // TODO import picture from camera
 // TODO modify picture (zoom, ...)
  * TODO after change type it is not refreshed on list
  * TODO implements copy of information field (and images?)
- * TODO allow to change password
  * TODO use hilton notifications and banner???
  * TODO check types on import
  * TODO sort after insert new card
- * TODO add warn message when import and export!
  * TODO multilanguage
  * TODO pc version?
- * TODO fix export and import to add field type
  * TODO fix problems with key
- * TODO remove/rename groupNames
- * TODO after insert first group, groups list is not refreshed
  */
 
 #include "eWallet.h"
@@ -38,8 +32,74 @@
 
 DatabaseManager * dbManager;
 
+
+
+RenameGroupDialog::RenameGroupDialog(QWidget *paTarget) : QDialog(paTarget)
+{
+	setContextMenuPolicy(Qt::NoContextMenu);
+	QHBoxLayout * layout = new QHBoxLayout(this);
+
+	layout->setMargin(0);
+	layout->setSpacing(0);
+
+	layout->setSpacing(10);
+	layout->setMargin(10);
+
+	QDialogButtonBox* dlgBtnBox = new QDialogButtonBox(this);
+	dlgBtnBox->setOrientation(Qt::Vertical);
+
+	QPushButton * closeButton = new QPushButton("Save && Close", this);
+	//closeButton->setFixedHeight(70);
+	closeButton->setContextMenuPolicy(Qt::NoContextMenu);
+	closeButton->setFocusPolicy(Qt::NoFocus);
+	closeButton->setIconSize(QSize(48, 48));
+	QObject::connect(closeButton, SIGNAL(clicked()), this, SLOT(save()));
+	dlgBtnBox->addButton(closeButton, QDialogButtonBox::ActionRole);
+
+	QGridLayout * componentLayout = new QGridLayout ();
+	componentLayout->setContentsMargins(15, 0, 15, 0);
+	componentLayout->setSpacing(5);
+
+	// Name label
+	QLabel * nameLabel = new QLabel("Name", this);
+	nameLabel->setContextMenuPolicy(Qt::NoContextMenu);
+	//nameLabel->setAlignment(Qt::AlignTop);
+	componentLayout->addWidget(nameLabel, 1, 1, 1, 2);
+
+// Name line edit
+	nameLineEdit = new QLineEdit(this);
+	nameLineEdit->setContextMenuPolicy(Qt::NoContextMenu);
+	//nameLineEdit->setAlignment(Qt::AlignTop);
+	componentLayout->addWidget(nameLineEdit, 1, 3, 1, 4);
+	layout->addLayout(componentLayout);
+	layout->setAlignment(componentLayout, Qt::AlignVCenter);
+
+	layout->addWidget(dlgBtnBox);
+	layout->setAlignment(dlgBtnBox, Qt::AlignBottom);
+
+	setLayout(layout);
+}
+
+void RenameGroupDialog::setName (const QString & name) {
+	nameLineEdit->setText (name);
+}
+
+QString RenameGroupDialog::getName () {
+	return nameLineEdit->text();
+}
+
+void RenameGroupDialog::save () {
+	//update ();
+	close();
+}
+RenameGroupDialog::~RenameGroupDialog() {
+
+}
+
+
 eWallet::eWallet(QString group, eWallet * _parent, QWidget *parent) : QMainWindow(parent)
 {
+	isFirst = false;
 	types = _parent->types;
 	fieldsByType = _parent->fieldsByType;
 
@@ -62,6 +122,7 @@ eWallet::eWallet(QString group, eWallet * _parent, QWidget *parent) : QMainWindo
 
 eWallet::eWallet(QWidget *parent) : QMainWindow(parent)
 {
+	isFirst = true;
 	currentGroup = "";
 	centralWidget = NULL;
 	isShowGroups = 0;
@@ -94,6 +155,7 @@ void eWallet::showList () {
 	qDebug () << "going to showList";
 
 	isAskPassword = false;
+	isChangingPassword = false;
 
 	setProperty("FingerScrollable", false);
 	centralWidget = new QWidget(this);
@@ -225,7 +287,7 @@ void eWallet::selectItem(QListWidgetItem * item) {
 			//extraWindow->setAttribute(Qt::WA_Maemo5StackedWindow);
 			extraWindow->setAttribute(Qt::WA_DeleteOnClose);
 	        connect(extraWindow, SIGNAL(destroyed(QObject*)),
-	                this, SLOT(extrawindowDestroyed(QObject*)));
+	                this, SLOT(windowDestroyedExtra(QObject*)));
 	        extraWindow->show();
 		}
 		//showTargets(reset, target);
@@ -356,7 +418,7 @@ void eWallet::deleteDB () {
 	messageBox = 0;
 }
 
-bool eWallet::openDataBase (QString pas)
+bool eWallet::openDataBase (const QString & pas)
 {
 	bool res = ::dbManager->openDB(pas);
 
@@ -553,6 +615,8 @@ void eWallet::showGroups(bool reset)
 {
 	qDebug () << "show Groups " << reset;
 
+	int current = fnListWidget->currentRow();
+
 	// Clean previous items from the layout
 	fnListWidget->clear();
 
@@ -583,7 +647,10 @@ void eWallet::showGroups(bool reset)
 
 	//filterBy(currentFilter);
 
-	fnListWidget->setCurrentRow(0);
+	if (current >= 0 && current <= grouplist.size ())
+		fnListWidget->setCurrentRow(current);
+	else
+		fnListWidget->setCurrentRow(0);
 
 	update();
 }
@@ -602,6 +669,8 @@ void eWallet::showTargets(bool reset, const QString & filter)
 {
 	qDebug () << "show Targets " << reset;
 
+	int current = fnListWidget->currentRow();
+
 	// Clean previous items from the layout
 	fnListWidget->clear();
 
@@ -638,22 +707,25 @@ void eWallet::createMenu()
 {
 	qDebug () << "create Menu " << isAskPassword;
 
+	menuBar()->clear();
 	if (isAskPassword) {
-		menuBar()->clear();
-
 		menuBar()->addAction("About", this, SLOT(about()));
-		menuBar()->addAction("Delete DB", this, SLOT(deleteDB()));
+		if (!isChangingPassword) {
+			menuBar()->addAction("Delete DB", this, SLOT(deleteDB()));
+		}
 		menuBar()->addAction("Exit", QApplication::instance(), SLOT(quit()));
 	}
 	else {
-		menuBar()->clear();
-
 		menuBar()->addAction("New", this, SLOT(newTarget()));
-		menuBar()->addAction("Edit", this, SLOT(_editcTarget()));
+		if (isShowGroups == 2)
+			menuBar()->addAction("Edit", this, SLOT(_editcTarget()));
 		menuBar()->addAction("Invert Sort", this, SLOT(sort()));
 		menuBar()->addAction("Import", this, SLOT(import()));
 		menuBar()->addAction("Export", this, SLOT(exportData()));
 		//menuBar()->addAction("Change View", this, SLOT(changeMode()));
+
+		menuBar()->addAction("Change Password", this, SLOT(changePassword()));
+
 		menuBar()->addAction("Exit", QApplication::instance(), SLOT(quit()));
 	}
 }
@@ -792,9 +864,9 @@ void eWallet::deleteTarget(Target * target) {
 	}
 }
 
-void eWallet::extrawindowDestroyed(QObject* obj) {
+void eWallet::windowDestroyedExtra(QObject* obj) {
     disconnect(obj, SIGNAL(destroyed(QObject*)),
-               this, SLOT(windowDestroyed(QObject*)));
+               this, SLOT(windowDestroyedExtra(QObject*)));
 
 	delete extraWindow;
 	extraWindow = 0;
@@ -809,15 +881,30 @@ void eWallet::windowDestroyed(QObject* obj)
     disconnect(obj, SIGNAL(destroyed(QObject*)),
                this, SLOT(windowDestroyed(QObject*)));
 
-    if (obj == detailWindow.data()) {
-		delete detailWindow;
-		detailWindow = 0;
-    }
-    else
-    if (obj == detailWindowEdit.data()) {
-		delete detailWindowEdit;
-		detailWindowEdit = 0;
-    }
+	delete detailWindow;
+	detailWindow = 0;
+ }
+void eWallet::windowDestroyedEdit(QObject* obj)
+{
+    disconnect(obj, SIGNAL(destroyed(QObject*)),
+               this, SLOT(windowDestroyedEdit(QObject*)));
+
+	delete detailWindowEdit;
+	detailWindowEdit = 0;
+
+	if (isShowGroups == 1) {
+		showGroups(true);
+	}
+	else {
+		grouplist.clear ();
+		grouplist = ::dbManager->getGroups ();
+
+		if (grouplist.size() >= 1 && isFirst) {
+			isShowGroups = 1;
+			showGroups(true);
+		}
+	}
+
 }
 
 void eWallet::viewTarget(Target * rent) {
@@ -849,7 +936,7 @@ void eWallet::editTarget(Target * rent) {
 		//detailWindowEdit->setAttribute(Qt::WA_Maemo5StackedWindow);
         detailWindowEdit->setAttribute(Qt::WA_DeleteOnClose);
         connect(detailWindowEdit, SIGNAL(destroyed(QObject*)),
-                this, SLOT(windowDestroyed(QObject*)));
+                this, SLOT(windowDestroyedEdit(QObject*)));
         detailWindowEdit->show();
 	}
 }
@@ -860,12 +947,11 @@ void eWallet::import() {
 	                this,
 	                "Choose import file",
 	                path,
-	                "Text (*.txt)");
+	                "Text files (*.txt);;");
 
 	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
 
 	if (!s.isNull () && !s.isEmpty ()) {
-
 		  QFile file(s);
 		  QString line;
 
@@ -892,7 +978,6 @@ void eWallet::import() {
 						  target->name = line.mid(j + 1).trimmed();
 						  break;
 					  case 1:
-						  // TODO check type with existing one!
 						  target->groupName = line.mid(j + 1).trimmed();
 						  break;
 					  case 2:
@@ -901,39 +986,74 @@ void eWallet::import() {
 						  break;
 					  default:
 						  TargetField * field = new TargetField ();
-						  field->name = line.mid (0, j).trimmed();
+
+				    	  int jj = line.mid (0, j).indexOf(QChar(';'));
+				    	  if (jj >= 0) {
+							  field->name = line.mid (0, jj).trimmed();
+
+			    			  int type = TYPE_STRING;
+			    			  switch (line.at(jj + 1).toAscii()) {
+								  case 'T':
+									  type = TYPE_TEXT;
+									  break;
+								  case 'D':
+									  type = TYPE_DATE;
+									  break;
+								  case 'U':
+									  type = TYPE_URL;
+									  break;
+								  case 'P':
+									  type = TYPE_PASSWORD;
+									  break;
+								  case 'N':
+									  type = TYPE_INTEGER;
+									  break;
+								  case 'H':
+									  type = TYPE_PHONE_NUMBER;
+									  break;
+								  case 'S':
+								  default:
+					    			  type = TYPE_STRING;
+			    			  }
+			    			  field->type = type;
+				    	  }
+				    	  else {
+							  field->name = line.mid (0, j).trimmed();
+
+//							  if (field->name.contains("url", Qt::CaseInsensitive)) {
+//								  field->type = TYPE_URL;
+//							  }
+//							  else
+//							  if (field->name.contains("password", Qt::CaseInsensitive)) {
+//								  field->type = TYPE_PASSWORD;
+//							  }
+//							  else
+//							  if (field->name.contains("pin", Qt::CaseInsensitive)) {
+//								  field->type = TYPE_PASSWORD;
+//							  }
+//							  else
+//							  if (field->name.contains("phone", Qt::CaseInsensitive)) {
+//								  field->type = TYPE_PHONE_NUMBER;
+//							  }
+//							  else
+//							  if (field->name.contains("number", Qt::CaseInsensitive)) {
+//								  field->type = TYPE_INTEGER;
+//							  }
+//							  else
+//							  if (field->name.contains("date", Qt::CaseInsensitive)) {
+//								  field->type = TYPE_DATE;
+//							  }
+//							  else
+//							  if (field->name.contains("note", Qt::CaseInsensitive)) {
+//								  field->type = TYPE_TEXT;
+//							  }
+//							  else
+								  field->type = TYPE_STRING;
+				    	  }
 						  field->value = line.mid(j + 1).trimmed();
-						  field->order = 10 * (i - 2);
+						  field->order = 10 * (i - 3);
 						  field->encrypted = true;
-						  if (field->name.contains("url", Qt::CaseInsensitive)) {
-							  field->type = TYPE_URL;
-						  }
-						  else
-						  if (field->name.contains("password", Qt::CaseInsensitive)) {
-							  field->type = TYPE_PASSWORD;
-						  }
-						  else
-						  if (field->name.contains("pin", Qt::CaseInsensitive)) {
-							  field->type = TYPE_PASSWORD;
-						  }
-						  else
-						  if (field->name.contains("phone", Qt::CaseInsensitive)) {
-							  field->type = TYPE_PHONE_NUMBER;
-						  }
-						  else
-						  if (field->name.contains("number", Qt::CaseInsensitive)) {
-							  field->type = TYPE_INTEGER;
-						  }
-						  else
-						  if (field->name.contains("date", Qt::CaseInsensitive)) {
-							  field->type = TYPE_DATE;
-						  }
-						  else
-						  if (field->name.contains("note", Qt::CaseInsensitive)) {
-							  field->type = TYPE_TEXT;
-						  }
-						  else
-							  field->type = TYPE_STRING;
+
 						  target->fields.append(field);
 					  }
 
@@ -949,6 +1069,40 @@ void eWallet::import() {
 		      else {
 		    	  if (target != NULL) {
 		    		  this->__insertTarget(target);
+		    		  {
+						QString imgPath = QFileInfo (s).absolutePath();
+						imgPath += "/";
+						imgPath += target->name.simplified().replace(' ', '_');
+						imgPath += ".png";
+
+						qDebug () << "checking image " << imgPath;
+
+						QFile imgFile (imgPath);
+						if (imgFile.exists()) {
+							QImage * currentImageN = new QImage ();
+							DetailWindow::loadImage(imgPath, *currentImageN); //;
+							target->setPicture(currentImageN);
+
+							::dbManager->updateTargetPicture (target);
+						}
+		    		  }
+		    		  {
+						QString imgPath = QFileInfo (s).absolutePath();
+						imgPath += "/";
+						imgPath += target->name.simplified().replace(' ', '_');
+						imgPath += "_BACK.png";
+
+						qDebug () << "checking image " << imgPath;
+
+						QFile imgFile (imgPath);
+						if (imgFile.exists()) {
+							QImage * currentImageN = new QImage ();
+							DetailWindow::loadImage(imgPath, *currentImageN); //;
+							target->setPictureBack(currentImageN);
+
+							::dbManager->updateTargetPictureBack (target);
+						}
+		    		  }
 		    	  }
 		    	  target = NULL;
 		    	  i = 0;
@@ -979,20 +1133,64 @@ void eWallet::newTarget() {
 		//detailWindowEdit->setAttribute(Qt::WA_Maemo5StackedWindow);
         detailWindowEdit->setAttribute(Qt::WA_DeleteOnClose);
         connect(detailWindowEdit, SIGNAL(destroyed(QObject*)),
-                this, SLOT(windowDestroyed(QObject*)));
+                this, SLOT(windowDestroyedEdit(QObject*)));
         detailWindowEdit->show();
 	}
 }
 
+void eWallet::renameGroup () {
+
+	if (fnListWidget->currentRow() > 0) {
+		QString target = grouplist.at(fnListWidget->currentRow() - 1);
+
+		RenameGroupDialog * dialog = new RenameGroupDialog (this);
+		dialog->setName(target);
+		dialog->exec();
+		QString newName = dialog->getName();
+
+		qDebug () << "NEW GROUP NAME " << target << " TO " << newName;
+
+		delete dialog;
+
+		if (::dbManager->renameGroup(target, newName)) {
+			showGroups(true);
+		}
+
+	}
+}
+
+void eWallet::deleteGroup () {
+	if (fnListWidget->currentRow() > 0) {
+		QString target = grouplist.at(fnListWidget->currentRow() - 1);
+
+		if (::dbManager->deleteGroup(target)) {
+			if (grouplist.size() > 1) {
+				showInfo(true);
+			}
+			else {
+				isShowGroups = 2;
+				showTargets(true, "");
+			}
+		}
+
+	}
+}
+
 void eWallet::contextMenuEvent (QContextMenuEvent * event) {
 	//qDebug () << "Should be show " << this->childAt(event->globalPos());
 
 	if (isShowGroups == 1) {
-
+		if (fnListWidget->currentRow() > 0) {
+			QMenu menu(this);
+			//menu.addAction("View", this, SLOT(_viewcTarget()));
+			menu.addAction("Rename", this, SLOT(renameGroup()));
+			menu.addAction("Delete", this, SLOT(deleteGroup()));
+			menu.exec(event->globalPos());
+		}
 	}
 	else {
 		if (fnListWidget->currentRow() >= 0) {
-		Target * target = list.at(fnListWidget->currentRow());
+			//Target * target = list.at(fnListWidget->currentRow());
 
 			QMenu menu(this);
 			menu.addAction("View", this, SLOT(_viewcTarget()));
@@ -1010,7 +1208,7 @@ void eWallet::exportData() {
 	                this,
 	                "Choose export file",
 	                path,
-	                "Text (*.txt)");
+	                "Text files (*.txt);;");
 
 	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
 	if (!s.isNull () && !s.isEmpty ()) {
@@ -1020,9 +1218,37 @@ void eWallet::exportData() {
 		    // file opened successfully
 		    QTextStream t(&file);        // use a text stream
 
+		    QList<Target *> list = ::dbManager->getTargets(NULL);
+
 			for (int i = 0; i < list.size (); i++) {
 				Target * target = list[i];
 
+				QImage * img = target->getPicture();
+
+				if (img != NULL) {
+					QString imgPath = QFileInfo (s).absolutePath();
+					imgPath += "/";
+					imgPath += target->name.simplified().replace(' ', '_');
+					imgPath += ".png";
+
+					qDebug () << "exporting image " << imgPath;
+
+					img->save(imgPath, "PNG"); // writes image into ba in PNG format
+				}
+
+				img = target->getPictureBack();
+
+				if (img != NULL) {
+					QString imgPath = QFileInfo (s).absolutePath();
+					imgPath += "/";
+					imgPath += target->name.simplified().replace(' ', '_');
+					imgPath += "_BACK.png";
+
+					qDebug () << "exporting back image " << imgPath;
+
+					img->save(imgPath, "PNG"); // writes image into ba in PNG format
+				}
+
 				t << "Name: " << target->name << "\n";
 				t << "Group Name: " << target->groupName << "\n";
 				t << "Type: " << target->type << "\n";
@@ -1031,17 +1257,43 @@ void eWallet::exportData() {
 
 				TargetField * field;
 				foreach(field, target->fields) {
-					t << field->name << ": " << field->value << "\n";
+	    			  switch (field->type) {
+						  case TYPE_TEXT:
+							t << field->name << ";T: " << field->value << "\n";
+							  break;
+						  case TYPE_DATE:
+								t << field->name << ";D: " << field->value << "\n";
+							  break;
+						  case TYPE_URL:
+								t << field->name << ";U: " << field->value << "\n";
+							  break;
+						  case TYPE_PASSWORD:
+								t << field->name << ";P: " << field->value << "\n";
+							  break;
+						  case TYPE_INTEGER:
+								t << field->name << ";N: " << field->value << "\n";
+							  break;
+						  case TYPE_PHONE_NUMBER:
+								t << field->name << ";H: " << field->value << "\n";
+							  break;
+						  case 'S':
+						  default:
+								t << field->name << ": " << field->value << "\n";
+	    			  }
 				}
 
 				t << "\n\n";
 
 				target->freeFieldList();
+				target->freeResources();
 
 				QApplication::instance()->processEvents();
 
+				delete target;
 			}
 
+			list.clear();
+
 		    // Close the file
 		    file.close();
 		  }
@@ -1060,17 +1312,38 @@ void eWallet::_newLikeTarget() {
 			//detailWindowEdit->setAttribute(Qt::WA_Maemo5StackedWindow);
 	        detailWindowEdit->setAttribute(Qt::WA_DeleteOnClose);
 	        connect(detailWindowEdit, SIGNAL(destroyed(QObject*)),
-	                this, SLOT(windowDestroyed(QObject*)));
+	                this, SLOT(windowDestroyedEdit(QObject*)));
 	        detailWindowEdit->show();
 		}
 	}
 }
 
 void eWallet::_editcTarget() {
-	if (fnListWidget->currentRow() >= 0) {
-		Target * target = list.at(fnListWidget->currentRow());
-		editTarget(target);
-	}
+//	if (isShowGroups == 1) {
+//		if (fnListWidget->currentRow() > 0) {
+//			QString target = grouplist.at(fnListWidget->currentRow() - 1);
+//
+//			RenameGroupDialog * dialog = new RenameGroupDialog (this);
+//			dialog->setName(target);
+//			dialog->exec();
+//			QString newName = dialog->getName();
+//
+//			qDebug () << "NEW GROUP NAME " << target << " TO " << newName;
+//
+//			delete dialog;
+//
+//			if (::dbManager->renameGroup(target, newName)) {
+//				showGroups(true);
+//			}
+//
+//		}
+//	}
+//	else {
+		if (fnListWidget->currentRow() >= 0) {
+			Target * target = list.at(fnListWidget->currentRow());
+			editTarget(target);
+		}
+//	}
 }
 void eWallet::_deletecTarget() {
 	if (fnListWidget->currentRow() >= 0) {
@@ -1094,27 +1367,70 @@ void eWallet::askPassword () {
 	times = 0;
 	
 	isAskPassword = true;
+	isChangingPassword = false;
 	createMenu();
 
 	if (loginDialogDlg.isNull()) {
 		loginDialogDlg = new LoginDialog(!::dbManager->hasPassword (), this);
-
-		// TODO should I remove all included widgets??
 		setCentralWidget(loginDialogDlg);
-		//show ();
 		setFocus ();
 		focusNextChild();
-
-//		centralWidget = NULL;
-//		vboxlayout = NULL;
-//		fnListWidget = NULL;
-//		inputWidget = NULL;
-//		cancelSearch = NULL;
-//		searchField = NULL;
-//		hideSearch = NULL;
 	}
 }
 
 void eWallet::about() {
-	QMessageBox::about(this, "About qtWallet", TEXT_ABOUT);
+	QMessageBox::about(this, "About Wallet", TEXT_ABOUT);
+}
+
+bool eWallet::finalPassword (const QString & pass) {
+	if (isChangingPassword)
+		reallyChangePassword(pass);
+	else
+		return openDataBase (pass);
+
+	return false;
+}
+
+void eWallet::changePassword () {
+	times = 0;
+	isChangingPassword = true;
+	isAskPassword = true;
+
+	createMenu();
+
+	if (loginDialogDlg.isNull()) {
+		loginDialogDlg = new LoginDialog(true, this);
+		setCentralWidget(loginDialogDlg);
+		setFocus ();
+		loginDialogDlg->setFocus();
+		focusNextChild();
+	}
+
+}
+
+void eWallet::reallyChangePassword (const QString & newPassword) {
+	showList ();
+
+	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
+
+	QMessageBox * messageBox = new QMessageBox (this);
+	messageBox->information(this, "Wallet", "Changing password, please wait until application quits", QMessageBox::Ok);
+
+	QString p = ::dbManager->changePassword(newPassword);
+
+	//QMessageBox * messageBox = new QMessageBox (this);
+	if (p.length() == 0)
+		messageBox->information(this, "Wallet", "Password changed successfully, application will exit", QMessageBox::Ok);
+	else {
+		QString message ("Password CANNOT BE changed, because ");
+		message.append (p);
+		message.append ("application will exit");
+		messageBox->information(this, "Wallet", message , QMessageBox::Ok);
+	}
+
+	delete messageBox;
+
+	delete ::dbManager;
+
+	exit (3);
 }
