@@ -14,6 +14,23 @@
 
 extern DatabaseManager * dbManager;
 
+class RenameGroupDialog: public QDialog {
+	Q_OBJECT
+
+public:
+	RenameGroupDialog (QWidget *parent = 0);
+	~RenameGroupDialog ();
+
+	void setName (const QString & name);
+	QString getName ();
+
+private slots:
+	void save ();
+
+private:
+	QLineEdit * nameLineEdit;
+};
+
 class eWallet: public QMainWindow
 {
 	Q_OBJECT
@@ -23,7 +40,8 @@ public:
 	eWallet (QString group, eWallet *_parent, QWidget *parent = 0);
 	~eWallet ();
 
-	bool openDataBase (QString pas);
+	bool finalPassword (const QString & pas);
+
 	bool _deleteTarget(Target* target);
 	bool _updateTarget(Target* target);
 	bool _updateTargetPicture(Target* target);
@@ -52,6 +70,7 @@ private:
 ////	void swipeTriggered (QSwipeGesture *gesture);
 //	bool gestureEvent (QGestureEvent *event);
 
+	bool openDataBase (const QString & pas);
 	void showList ();
 	void keyPressEvent(QKeyEvent*);
 	void createMenu();
@@ -62,12 +81,16 @@ private:
 	void filterBy (const QString & s);
 	QPixmap getImage (Target * f);
 	bool isAskPassword;
+	bool isChangingPassword;
+    void reallyChangePassword (const QString & newPassword);
 
 private slots:
 	void orientationChanged(int);
     void windowDestroyed(QObject* obj = 0);
-    void extrawindowDestroyed(QObject* obj = 0);
+    void windowDestroyedExtra(QObject* obj = 0);
+    void windowDestroyedEdit(QObject* obj = 0);
     void sort ();
+    void changePassword ();
     void _newLikeTarget ();
 	void _editcTarget();
 	void _clickBack();
@@ -81,11 +104,13 @@ private slots:
 	void exportData();
 	void textChanged (const QString &);
 	void contextMenuEvent (QContextMenuEvent * event);
+	void renameGroup ();
+	void deleteGroup ();
 	
 private:
 	void askPassword ();
 	void closeEvent(QCloseEvent *event);
-	
+	bool isFirst;
 	QVBoxLayout * vboxlayout;
 	int times;
 	QListWidget* fnListWidget;
